# Generated from porter.sbl by Snowball 3.1.0 - https://snowballstem.org/

from .basestemmer import BaseStemmer
from .among import Among


class PorterStemmer(BaseStemmer):
    '''
    This class implements the stemming algorithm defined by a snowball script.
    Generated from porter.sbl by Snowball 3.1.0 - https://snowballstem.org/
    '''

    g_v = {"a", "e", "i", "o", "u", "y"}

    g_v_WXY = {"Y", "a", "e", "i", "o", "u", "w", "x", "y"}

    I_p2 = 0
    I_p1 = 0

    def __r_shortv(self):
        if not self.out_grouping_b(PorterStemmer.g_v_WXY):
            return False
        if not self.in_grouping_b(PorterStemmer.g_v):
            return False
        return self.out_grouping_b(PorterStemmer.g_v)

    def __r_R1(self):
        return self.I_p1 <= self.cursor

    def __r_R2(self):
        return self.I_p2 <= self.cursor

    def __r_Step_1a(self):
        self.ket = self.cursor
        among_var = self.find_among_b(PorterStemmer.a_0)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if among_var == 1:
            self.slice_from("ss")
        elif among_var == 2:
            self.slice_from("i")
        elif among_var == 3:
            self.slice_del()
        return True

    def __r_Step_1b(self):
        self.ket = self.cursor
        among_var = self.find_among_b(PorterStemmer.a_2)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if among_var == 1:
            if not self.__r_R1():
                return False
            self.slice_from("ee")
        else:
            v_1 = self.limit - self.cursor
            if not self.go_out_grouping_b(PorterStemmer.g_v):
                return False
            self.cursor -= 1
            self.cursor = self.limit - v_1
            self.slice_del()
            v_2 = self.limit - self.cursor
            among_var = self.find_among_b(PorterStemmer.a_1)
            self.cursor = self.limit - v_2
            if among_var == 1:
                c = self.cursor
                self.insert(self.cursor, self.cursor, "e")
                self.cursor = c
            elif among_var == 2:
                self.ket = self.cursor
                if self.cursor <= self.limit_backward:
                    return False
                self.cursor -= 1
                self.bra = self.cursor
                self.slice_del()
            else:
                if self.cursor != self.I_p1:
                    return False
                v_3 = self.limit - self.cursor
                if not self.__r_shortv():
                    return False
                self.cursor = self.limit - v_3
                c = self.cursor
                self.insert(self.cursor, self.cursor, "e")
                self.cursor = c
        return True

    def __r_Step_1c(self):
        self.ket = self.cursor
        while True:
            try:
                if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "y":
                    raise lab0()
                self.cursor -= 1
                break
            except lab0: pass
            if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "Y":
                return False
            self.cursor -= 1
            break
        self.bra = self.cursor
        if not self.go_out_grouping_b(PorterStemmer.g_v):
            return False
        self.cursor -= 1
        self.slice_from("i")
        return True

    def __r_Step_2(self):
        self.ket = self.cursor
        among_var = self.find_among_b(PorterStemmer.a_3)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if not self.__r_R1():
            return False
        self.slice_from(PorterStemmer.as_3[among_var - 1])
        return True

    def __r_Step_3(self):
        self.ket = self.cursor
        among_var = self.find_among_b(PorterStemmer.a_4)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if not self.__r_R1():
            return False
        if among_var == 1:
            self.slice_from("al")
        elif among_var == 2:
            self.slice_from("ic")
        else:
            self.slice_del()
        return True

    def __r_Step_4(self):
        self.ket = self.cursor
        among_var = self.find_among_b(PorterStemmer.a_5)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if not self.__r_R2():
            return False
        if among_var == 1:
            self.slice_del()
        else:
            while True:
                try:
                    if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "s":
                        raise lab0()
                    self.cursor -= 1
                    break
                except lab0: pass
                if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "t":
                    return False
                self.cursor -= 1
                break
            self.slice_del()
        return True

    def __r_Step_5a(self):
        self.ket = self.cursor
        if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "e":
            return False
        self.cursor -= 1
        self.bra = self.cursor
        while True:
            try:
                if not self.__r_R2():
                    raise lab0()
                break
            except lab0: pass
            if not self.__r_R1():
                return False
            v_1 = self.limit - self.cursor
            try:
                if not self.__r_shortv():
                    raise lab0()
                return False
            except lab0: pass
            self.cursor = self.limit - v_1
            break
        self.slice_del()
        return True

    def __r_Step_5b(self):
        self.ket = self.cursor
        if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "l":
            return False
        self.cursor -= 1
        self.bra = self.cursor
        if not self.__r_R2():
            return False
        if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "l":
            return False
        self.cursor -= 1
        self.slice_del()
        return True

    def _stem(self):
        B_Y_found = False
        v_1 = self.cursor
        try:
            self.bra = self.cursor
            if self.cursor == self.limit or self.current[self.cursor] != "y":
                raise lab0()
            self.cursor += 1
            self.ket = self.cursor
            self.slice_from("Y")
            B_Y_found = True
        except lab0: pass
        self.cursor = v_1
        v_2 = self.cursor
        try:
            while True:
                v_3 = self.cursor
                try:
                    while True:
                        v_4 = self.cursor
                        try:
                            if not self.in_grouping(PorterStemmer.g_v):
                                raise lab2()
                            self.bra = self.cursor
                            if self.cursor == self.limit or self.current[self.cursor] != "y":
                                raise lab2()
                            self.cursor += 1
                            self.ket = self.cursor
                            self.cursor = v_4
                            break
                        except lab2: pass
                        self.cursor = v_4
                        if self.cursor >= self.limit:
                            raise lab1()
                        self.cursor += 1
                    self.slice_from("Y")
                    B_Y_found = True
                    continue
                except lab1: pass
                self.cursor = v_3
                break
        except lab0: pass
        self.cursor = v_2
        self.I_p1 = self.limit
        self.I_p2 = self.limit
        v_5 = self.cursor
        try:
            if not self.go_out_grouping(PorterStemmer.g_v):
                raise lab0()
            self.cursor += 1
            if not self.go_in_grouping(PorterStemmer.g_v):
                raise lab0()
            self.cursor += 1
            self.I_p1 = self.cursor
            if not self.go_out_grouping(PorterStemmer.g_v):
                raise lab0()
            self.cursor += 1
            if not self.go_in_grouping(PorterStemmer.g_v):
                raise lab0()
            self.cursor += 1
            self.I_p2 = self.cursor
        except lab0: pass
        self.cursor = v_5
        self.limit_backward = self.cursor
        self.cursor = self.limit
        v_6 = self.limit - self.cursor
        self.__r_Step_1a()
        self.cursor = self.limit - v_6
        v_7 = self.limit - self.cursor
        self.__r_Step_1b()
        self.cursor = self.limit - v_7
        v_8 = self.limit - self.cursor
        self.__r_Step_1c()
        self.cursor = self.limit - v_8
        v_9 = self.limit - self.cursor
        self.__r_Step_2()
        self.cursor = self.limit - v_9
        v_10 = self.limit - self.cursor
        self.__r_Step_3()
        self.cursor = self.limit - v_10
        v_11 = self.limit - self.cursor
        self.__r_Step_4()
        self.cursor = self.limit - v_11
        v_12 = self.limit - self.cursor
        self.__r_Step_5a()
        self.cursor = self.limit - v_12
        v_13 = self.limit - self.cursor
        self.__r_Step_5b()
        self.cursor = self.limit - v_13
        self.cursor = self.limit_backward
        v_14 = self.cursor
        try:
            if not B_Y_found:
                raise lab0()
            while True:
                v_15 = self.cursor
                try:
                    while True:
                        v_16 = self.cursor
                        try:
                            self.bra = self.cursor
                            if self.cursor == self.limit or self.current[self.cursor] != "Y":
                                raise lab2()
                            self.cursor += 1
                            self.ket = self.cursor
                            self.cursor = v_16
                            break
                        except lab2: pass
                        self.cursor = v_16
                        if self.cursor >= self.limit:
                            raise lab1()
                        self.cursor += 1
                    self.slice_from("y")
                    continue
                except lab1: pass
                self.cursor = v_15
                break
        except lab0: pass
        self.cursor = v_14
        return True

    a_0 = [
        Among("s", -1, 3),
        Among("ies", 0, 2),
        Among("sses", 0, 1),
        Among("ss", 0, -1)
    ]

    a_1 = [
        Among("", -1, 3),
        Among("bb", 0, 2),
        Among("dd", 0, 2),
        Among("ff", 0, 2),
        Among("gg", 0, 2),
        Among("bl", 0, 1),
        Among("mm", 0, 2),
        Among("nn", 0, 2),
        Among("pp", 0, 2),
        Among("rr", 0, 2),
        Among("at", 0, 1),
        Among("tt", 0, 2),
        Among("iz", 0, 1)
    ]

    a_2 = [
        Among("ed", -1, 2),
        Among("eed", 0, 1),
        Among("ing", -1, 2)
    ]

    a_3 = [
        Among("anci", -1, 3),
        Among("enci", -1, 2),
        Among("abli", -1, 4),
        Among("eli", -1, 6),
        Among("alli", -1, 9),
        Among("ousli", -1, 11),
        Among("entli", -1, 5),
        Among("aliti", -1, 9),
        Among("biliti", -1, 13),
        Among("iviti", -1, 12),
        Among("tional", -1, 1),
        Among("ational", 10, 8),
        Among("alism", -1, 9),
        Among("ation", -1, 8),
        Among("ization", 13, 7),
        Among("izer", -1, 7),
        Among("ator", -1, 8),
        Among("iveness", -1, 12),
        Among("fulness", -1, 10),
        Among("ousness", -1, 11)
    ]
    as_3 = ("tion", "ence", "ance", "able", "ent", "e", "ize", "ate", "al", "ful", "ous", "ive", "ble")

    a_4 = [
        Among("icate", -1, 2),
        Among("ative", -1, 3),
        Among("alize", -1, 1),
        Among("iciti", -1, 2),
        Among("ical", -1, 2),
        Among("ful", -1, 3),
        Among("ness", -1, 3)
    ]

    a_5 = [
        Among("ic", -1, 1),
        Among("ance", -1, 1),
        Among("ence", -1, 1),
        Among("able", -1, 1),
        Among("ible", -1, 1),
        Among("ate", -1, 1),
        Among("ive", -1, 1),
        Among("ize", -1, 1),
        Among("iti", -1, 1),
        Among("al", -1, 1),
        Among("ism", -1, 1),
        Among("ion", -1, 2),
        Among("er", -1, 1),
        Among("ous", -1, 1),
        Among("ant", -1, 1),
        Among("ent", -1, 1),
        Among("ment", 15, 1),
        Among("ement", 16, 1),
        Among("ou", -1, 1)
    ]


class lab0(BaseException): pass


class lab1(BaseException): pass


class lab2(BaseException): pass



# Generated from german.sbl by Snowball 3.1.0 - https://snowballstem.org/

from .basestemmer import BaseStemmer
from .among import Among


class GermanStemmer(BaseStemmer):
    '''
    This class implements the stemming algorithm defined by a snowball script.
    Generated from german.sbl by Snowball 3.1.0 - https://snowballstem.org/
    '''

    g_v = {"a", "e", "i", "o", "u", "y", "ä", "ö", "ü"}

    g_et_ending = {"U", "d", "f", "g", "k", "l", "m", "n", "r", "s", "t", "z", "ä"}

    g_s_ending = {"b", "d", "f", "g", "h", "k", "l", "m", "n", "r", "t"}

    g_st_ending = {"b", "d", "f", "g", "h", "k", "l", "m", "n", "t"}

    I_p2 = 0
    I_p1 = 0

    def __r_prelude(self):
        v_1 = self.cursor
        while True:
            v_2 = self.cursor
            try:
                while True:
                    v_3 = self.cursor
                    try:
                        if not self.in_grouping(GermanStemmer.g_v):
                            raise lab1()
                        self.bra = self.cursor
                        while True:
                            v_4 = self.cursor
                            try:
                                if self.cursor == self.limit or self.current[self.cursor] != "u":
                                    raise lab2()
                                self.cursor += 1
                                self.ket = self.cursor
                                if not self.in_grouping(GermanStemmer.g_v):
                                    raise lab2()
                                self.slice_from("U")
                                break
                            except lab2: pass
                            self.cursor = v_4
                            if self.cursor == self.limit or self.current[self.cursor] != "y":
                                raise lab1()
                            self.cursor += 1
                            self.ket = self.cursor
                            if not self.in_grouping(GermanStemmer.g_v):
                                raise lab1()
                            self.slice_from("Y")
                            break
                        self.cursor = v_3
                        break
                    except lab1: pass
                    self.cursor = v_3
                    if self.cursor >= self.limit:
                        raise lab0()
                    self.cursor += 1
                continue
            except lab0: pass
            self.cursor = v_2
            break
        self.cursor = v_1
        while True:
            v_5 = self.cursor
            try:
                self.bra = self.cursor
                among_var = self.find_among(GermanStemmer.a_0)
                self.ket = self.cursor
                if among_var == 1:
                    self.slice_from("ss")
                elif among_var == 2:
                    self.slice_from("ä")
                elif among_var == 3:
                    self.slice_from("ö")
                elif among_var == 4:
                    self.slice_from("ü")
                elif among_var == 5:
                    if self.cursor >= self.limit:
                        raise lab0()
                    self.cursor += 1
                continue
            except lab0: pass
            self.cursor = v_5
            break
        return True

    def __r_mark_regions(self):
        self.I_p1 = self.limit
        self.I_p2 = self.limit
        v_1 = self.cursor
        if self.cursor + 3 > self.limit:
            return False
        self.cursor += 3
        I_x = self.cursor
        self.cursor = v_1
        if not self.go_out_grouping(GermanStemmer.g_v):
            return False
        self.cursor += 1
        if not self.go_in_grouping(GermanStemmer.g_v):
            return False
        self.cursor += 1
        self.I_p1 = self.cursor
        try:
            if self.I_p1 >= I_x:
                raise lab0()
            self.I_p1 = I_x
        except lab0: pass
        if not self.go_out_grouping(GermanStemmer.g_v):
            return False
        self.cursor += 1
        if not self.go_in_grouping(GermanStemmer.g_v):
            return False
        self.cursor += 1
        self.I_p2 = self.cursor
        return True

    def __r_postlude(self):
        while True:
            v_1 = self.cursor
            try:
                self.bra = self.cursor
                among_var = self.find_among(GermanStemmer.a_1)
                self.ket = self.cursor
                if among_var == 1:
                    self.slice_from("y")
                elif among_var == 2:
                    self.slice_from("u")
                elif among_var == 3:
                    self.slice_from("a")
                elif among_var == 4:
                    self.slice_from("o")
                else:
                    if self.cursor >= self.limit:
                        raise lab0()
                    self.cursor += 1
                continue
            except lab0: pass
            self.cursor = v_1
            break
        return True

    def __r_R1(self):
        return self.I_p1 <= self.cursor

    def __r_R2(self):
        return self.I_p2 <= self.cursor

    def __r_standard_suffix(self):
        v_1 = self.limit - self.cursor
        try:
            self.ket = self.cursor
            among_var = self.find_among_b(GermanStemmer.a_2)
            if among_var == 0:
                raise lab0()
            self.bra = self.cursor
            if not self.__r_R1():
                raise lab0()
            if among_var == 1:
                try:
                    if not self.eq_s_b("syst"):
                        raise lab1()
                    raise lab0()
                except lab1: pass
                self.slice_del()
            elif among_var == 2:
                self.slice_del()
            elif among_var == 3:
                self.slice_del()
                v_2 = self.limit - self.cursor
                try:
                    self.ket = self.cursor
                    if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "s":
                        self.cursor = self.limit - v_2
                        raise lab1()
                    self.cursor -= 1
                    self.bra = self.cursor
                    if not self.eq_s_b("nis"):
                        self.cursor = self.limit - v_2
                        raise lab1()
                    self.slice_del()
                except lab1: pass
            elif among_var == 4:
                if not self.in_grouping_b(GermanStemmer.g_s_ending):
                    raise lab0()
                self.slice_del()
            else:
                self.slice_from("l")
        except lab0: pass
        self.cursor = self.limit - v_1
        v_3 = self.limit - self.cursor
        try:
            self.ket = self.cursor
            among_var = self.find_among_b(GermanStemmer.a_4)
            if among_var == 0:
                raise lab0()
            self.bra = self.cursor
            if not self.__r_R1():
                raise lab0()
            if among_var == 1:
                self.slice_del()
            elif among_var == 2:
                if not self.in_grouping_b(GermanStemmer.g_st_ending):
                    raise lab0()
                if self.cursor - 3 < self.limit_backward:
                    raise lab0()
                self.cursor -= 3
                self.slice_del()
            else:
                v_4 = self.limit - self.cursor
                if not self.in_grouping_b(GermanStemmer.g_et_ending):
                    raise lab0()
                self.cursor = self.limit - v_4
                v_5 = self.limit - self.cursor
                try:
                    if self.find_among_b(GermanStemmer.a_3) == 0:
                        raise lab1()
                    raise lab0()
                except lab1: pass
                self.cursor = self.limit - v_5
                self.slice_del()
        except lab0: pass
        self.cursor = self.limit - v_3
        v_6 = self.limit - self.cursor
        try:
            self.ket = self.cursor
            among_var = self.find_among_b(GermanStemmer.a_6)
            if among_var == 0:
                raise lab0()
            self.bra = self.cursor
            if not self.__r_R2():
                raise lab0()
            if among_var == 1:
                self.slice_del()
                v_7 = self.limit - self.cursor
                try:
                    self.ket = self.cursor
                    if not self.eq_s_b("ig"):
                        self.cursor = self.limit - v_7
                        raise lab1()
                    self.bra = self.cursor
                    try:
                        if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "e":
                            raise lab2()
                        self.cursor -= 1
                        self.cursor = self.limit - v_7
                        raise lab1()
                    except lab2: pass
                    if not self.__r_R2():
                        self.cursor = self.limit - v_7
                        raise lab1()
                    self.slice_del()
                except lab1: pass
            elif among_var == 2:
                try:
                    if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "e":
                        raise lab1()
                    self.cursor -= 1
                    raise lab0()
                except lab1: pass
                self.slice_del()
            elif among_var == 3:
                self.slice_del()
                v_8 = self.limit - self.cursor
                try:
                    self.ket = self.cursor
                    while True:
                        try:
                            if not self.eq_s_b("er"):
                                raise lab2()
                            break
                        except lab2: pass
                        if not self.eq_s_b("en"):
                            self.cursor = self.limit - v_8
                            raise lab1()
                        break
                    self.bra = self.cursor
                    if not self.__r_R1():
                        self.cursor = self.limit - v_8
                        raise lab1()
                    self.slice_del()
                except lab1: pass
            else:
                self.slice_del()
                v_9 = self.limit - self.cursor
                try:
                    self.ket = self.cursor
                    if self.find_among_b(GermanStemmer.a_5) == 0:
                        self.cursor = self.limit - v_9
                        raise lab1()
                    self.bra = self.cursor
                    if not self.__r_R2():
                        self.cursor = self.limit - v_9
                        raise lab1()
                    self.slice_del()
                except lab1: pass
        except lab0: pass
        self.cursor = self.limit - v_6
        v_10 = self.limit - self.cursor
        try:
            self.ket = self.cursor
            if self.find_among_b(GermanStemmer.a_7) == 0:
                raise lab0()
            self.bra = self.cursor
            if self.cursor <= self.limit_backward:
                raise lab0()
            self.cursor -= 1
            try:
                if self.cursor > self.limit_backward:
                    raise lab1()
                raise lab0()
            except lab1: pass
            self.slice_del()
        except lab0: pass
        self.cursor = self.limit - v_10
        return True

    def _stem(self):
        v_1 = self.cursor
        self.__r_prelude()
        self.cursor = v_1
        v_2 = self.cursor
        self.__r_mark_regions()
        self.cursor = v_2
        self.limit_backward = self.cursor
        self.cursor = self.limit
        self.__r_standard_suffix()
        self.cursor = self.limit_backward
        v_3 = self.cursor
        self.__r_postlude()
        self.cursor = v_3
        return True

    a_0 = [
        Among("", -1, 5),
        Among("ae", 0, 2),
        Among("oe", 0, 3),
        Among("qu", 0, -1),
        Among("ue", 0, 4),
        Among("ß", 0, 1)
    ]

    a_1 = [
        Among("", -1, 5),
        Among("U", 0, 2),
        Among("Y", 0, 1),
        Among("ä", 0, 3),
        Among("ö", 0, 4),
        Among("ü", 0, 2)
    ]

    a_2 = [
        Among("e", -1, 3),
        Among("em", -1, 1),
        Among("en", -1, 3),
        Among("erinnen", 2, 2),
        Among("erin", -1, 2),
        Among("ln", -1, 5),
        Among("ern", -1, 2),
        Among("er", -1, 2),
        Among("s", -1, 4),
        Among("es", 8, 3),
        Among("lns", 8, 5)
    ]

    a_3 = [
        Among("tick", -1, -1),
        Among("plan", -1, -1),
        Among("geordn", -1, -1),
        Among("intern", -1, -1),
        Among("tr", -1, -1)
    ]

    a_4 = [
        Among("en", -1, 1),
        Among("er", -1, 1),
        Among("et", -1, 3),
        Among("st", -1, 2),
        Among("est", 3, 1)
    ]

    a_5 = [
        Among("ig", -1, 1),
        Among("lich", -1, 1)
    ]

    a_6 = [
        Among("end", -1, 1),
        Among("ig", -1, 2),
        Among("ung", -1, 1),
        Among("lich", -1, 3),
        Among("isch", -1, 2),
        Among("ik", -1, 2),
        Among("heit", -1, 3),
        Among("keit", -1, 4)
    ]

    a_7 = [
        Among("'", -1, 1),
        Among("'sch", -1, 1),
        Among("'s", -1, 1)
    ]


class lab0(BaseException): pass


class lab1(BaseException): pass


class lab2(BaseException): pass



# Generated from polish.sbl by Snowball 3.1.0 - https://snowballstem.org/

from .basestemmer import BaseStemmer
from .among import Among


class PolishStemmer(BaseStemmer):
    '''
    This class implements the stemming algorithm defined by a snowball script.
    Generated from polish.sbl by Snowball 3.1.0 - https://snowballstem.org/
    '''

    g_v = {"a", "e", "i", "o", "u", "y", "ó", "ą", "ę"}

    I_p1 = 0

    def __r_mark_regions(self):
        self.I_p1 = self.limit
        if not self.go_out_grouping(PolishStemmer.g_v):
            return False
        self.cursor += 1
        if not self.go_in_grouping(PolishStemmer.g_v):
            return False
        self.cursor += 1
        self.I_p1 = self.cursor
        return True

    def __r_R1(self):
        return self.I_p1 <= self.cursor

    def __r_remove_endings(self):
        v_1 = self.limit - self.cursor
        try:
            if self.cursor < self.I_p1:
                raise lab0()
            v_3 = self.limit_backward
            self.limit_backward = self.I_p1
            self.ket = self.cursor
            if self.find_among_b(PolishStemmer.a_0) == 0:
                self.limit_backward = v_3
                raise lab0()
            self.bra = self.cursor
            self.limit_backward = v_3
            self.slice_del()
        except lab0: pass
        self.cursor = self.limit - v_1
        self.ket = self.cursor
        among_var = self.find_among_b(PolishStemmer.a_2)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if among_var == 1:
            self.slice_del()
        elif among_var == 2:
            self.slice_from("s")
        elif among_var == 3:
            while True:
                v_4 = self.limit - self.cursor
                try:
                    if not self.__r_R1():
                        raise lab0()
                    self.slice_del()
                    break
                except lab0: pass
                self.cursor = self.limit - v_4
                self.slice_from("s")
                break
        elif among_var == 4:
            self.slice_from("ł")
        else:
            self.slice_del()
            v_5 = self.limit - self.cursor
            try:
                self.ket = self.cursor
                among_var = self.find_among_b(PolishStemmer.a_1)
                if among_var == 0:
                    self.cursor = self.limit - v_5
                    raise lab0()
                self.bra = self.cursor
                if among_var == 1:
                    self.slice_del()
                else:
                    self.slice_from("s")
            except lab0: pass
        v_6 = self.limit - self.cursor
        try:
            self.ket = self.cursor
            if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "'":
                self.cursor = self.limit - v_6
                raise lab0()
            self.cursor -= 1
            self.bra = self.cursor
            self.slice_del()
        except lab0: pass
        return True

    def __r_normalize_consonant(self):
        self.ket = self.cursor
        among_var = self.find_among_b(PolishStemmer.a_3)
        if among_var == 0:
            return False
        self.bra = self.cursor
        try:
            if self.cursor > self.limit_backward:
                raise lab0()
            return False
        except lab0: pass
        self.slice_from(PolishStemmer.as_3[among_var - 1])
        return True

    def _stem(self):
        v_1 = self.cursor
        self.__r_mark_regions()
        self.cursor = v_1
        while True:
            v_2 = self.cursor
            try:
                if self.cursor + 2 > self.limit:
                    raise lab0()
                self.cursor += 2
                self.limit_backward = self.cursor
                self.cursor = self.limit
                if not self.__r_remove_endings():
                    raise lab0()
                self.cursor = self.limit_backward
                break
            except lab0: pass
            self.cursor = v_2
            self.limit_backward = self.cursor
            self.cursor = self.limit
            if not self.__r_normalize_consonant():
                return False
            self.cursor = self.limit_backward
            break
        return True

    a_0 = [
        Among("byście", -1, 1),
        Among("bym", -1, 1),
        Among("by", -1, 1),
        Among("byśmy", -1, 1),
        Among("byś", -1, 1)
    ]

    a_1 = [
        Among("ąc", -1, 1),
        Among("ając", 0, 1),
        Among("sząc", 0, 2),
        Among("sz", -1, 1),
        Among("iejsz", 3, 1)
    ]

    a_2 = [
        Among("a", -1, 1, __r_R1),
        Among("ąca", 0, 1),
        Among("ająca", 1, 1),
        Among("sząca", 1, 2),
        Among("ia", 0, 1, __r_R1),
        Among("sza", 0, 1),
        Among("iejsza", 5, 1),
        Among("ała", 0, 1),
        Among("iała", 7, 1),
        Among("iła", 0, 1),
        Among("ąc", -1, 1),
        Among("ając", 10, 1),
        Among("e", -1, 1, __r_R1),
        Among("ące", 12, 1),
        Among("ające", 13, 1),
        Among("szące", 13, 2),
        Among("ie", 12, 1, __r_R1),
        Among("cie", 16, 1),
        Among("acie", 17, 1),
        Among("ecie", 17, 1),
        Among("icie", 17, 1),
        Among("ajcie", 17, 1),
        Among("liście", 17, 4),
        Among("aliście", 22, 1),
        Among("ieliście", 22, 1),
        Among("iliście", 22, 1),
        Among("łyście", 17, 4),
        Among("ałyście", 26, 1),
        Among("iałyście", 27, 1),
        Among("iłyście", 26, 1),
        Among("sze", 12, 1),
        Among("iejsze", 30, 1),
        Among("ach", -1, 1, __r_R1),
        Among("iach", 32, 1, __r_R1),
        Among("ich", -1, 5),
        Among("ych", -1, 5),
        Among("i", -1, 1, __r_R1),
        Among("ali", 36, 1),
        Among("ieli", 36, 1),
        Among("ili", 36, 1),
        Among("ami", 36, 1, __r_R1),
        Among("iami", 40, 1, __r_R1),
        Among("imi", 36, 5),
        Among("ymi", 36, 5),
        Among("owi", 36, 1, __r_R1),
        Among("iowi", 44, 1, __r_R1),
        Among("aj", -1, 1),
        Among("ej", -1, 5),
        Among("iej", 47, 5),
        Among("am", -1, 1),
        Among("ałam", 49, 1),
        Among("iałam", 50, 1),
        Among("iłam", 49, 1),
        Among("em", -1, 1, __r_R1),
        Among("iem", 53, 1, __r_R1),
        Among("ałem", 53, 1),
        Among("iałem", 55, 1),
        Among("iłem", 53, 1),
        Among("im", -1, 5),
        Among("om", -1, 1, __r_R1),
        Among("iom", 59, 1, __r_R1),
        Among("ym", -1, 5),
        Among("o", -1, 1, __r_R1),
        Among("ego", 62, 5),
        Among("iego", 63, 5),
        Among("ało", 62, 1),
        Among("iało", 65, 1),
        Among("iło", 62, 1),
        Among("u", -1, 1, __r_R1),
        Among("iu", 68, 1, __r_R1),
        Among("emu", 68, 5),
        Among("iemu", 70, 5),
        Among("ów", -1, 1, __r_R1),
        Among("y", -1, 5),
        Among("amy", 73, 1),
        Among("emy", 73, 1),
        Among("imy", 73, 1),
        Among("liśmy", 73, 4),
        Among("aliśmy", 77, 1),
        Among("ieliśmy", 77, 1),
        Among("iliśmy", 77, 1),
        Among("łyśmy", 73, 4),
        Among("ałyśmy", 81, 1),
        Among("iałyśmy", 82, 1),
        Among("iłyśmy", 81, 1),
        Among("ały", 73, 1),
        Among("iały", 85, 1),
        Among("iły", 73, 1),
        Among("asz", -1, 1),
        Among("esz", -1, 1),
        Among("isz", -1, 1),
        Among("ą", -1, 1, __r_R1),
        Among("ącą", 91, 1),
        Among("ającą", 92, 1),
        Among("szącą", 92, 2),
        Among("ią", 91, 1, __r_R1),
        Among("ają", 91, 1),
        Among("szą", 91, 3),
        Among("iejszą", 97, 1),
        Among("ać", -1, 1),
        Among("ieć", -1, 1),
        Among("ić", -1, 1),
        Among("ąć", -1, 1),
        Among("aść", -1, 1),
        Among("eść", -1, 1),
        Among("ę", -1, 1),
        Among("szę", 105, 2),
        Among("ał", -1, 1),
        Among("iał", 107, 1),
        Among("ił", -1, 1),
        Among("łaś", -1, 4),
        Among("ałaś", 110, 1),
        Among("iałaś", 111, 1),
        Among("iłaś", 110, 1),
        Among("łeś", -1, 4),
        Among("ałeś", 114, 1),
        Among("iałeś", 115, 1),
        Among("iłeś", 114, 1)
    ]

    a_3 = [
        Among("ć", -1, 1),
        Among("ń", -1, 2),
        Among("ś", -1, 3),
        Among("ź", -1, 4)
    ]
    as_3 = ("c", "n", "s", "z")


class lab0(BaseException): pass
